
--local Dominos_BuffTimes = Dominos_BuffTimes

--[[ Configuration functions ]]--

Dominos_BuffTimes.ignoreOptions = {
	addspell = {
		name = "Remove Spell",
		desc = "Remove spell from ignored list.",
		type = "execute",
		func = "RemoveSpellFromIgnored",
		order = 200,
	},
}

Dominos_BuffTimes.yourBuffOptions = {
	addspell = {
		name = "Remove Spell",
		desc = "Remove spell from player buffs list.",
		type = "execute",
		func = "RemoveSpellFromYourBuff",
		order = 200,
	},
}

Dominos_BuffTimes.allDebuffOptions = {
	addspell = {
		name = "Remove Spell",
		desc = "Remove spell from any debuff list.",
		type = "execute",
		func = "RemoveSpellFromAnyDebuff",
		order = 200,
	},
}

function Dominos_BuffTimes:InitOptions()
	
	self.options = {
		desc = "Dominos_BuffTimes - (de)buff timers on buttons",
		type = "group",
		childGroups = "tab",
		handler = Dominos_BuffTimes,
		
		args = {
			ignored = {
				name = "Ignored",
				desc = "Spells for which timers will not be shown.",
				type = "group",
				
				args = {
					description = {
						type = "description",
						name = "The following spells will not show times on the buttons (borders will still be colored).",
						order = 1,
					},
					spellname = {
						name = "Spell to add",
						type = "input",
						desc = "Name of the spell to add to the ignored list.",
						set = "AddSpellToIgnored",
						order = 100,
					}, 
				},
			},
			yourbuffs = {
				name = "Player only buffs",
				desc = "Buffs for which timers will only be shown if you casted it.",
				type = "group",
				
				args = {
					description = {
						type = "description",
						name = "By default, all buffs will show times for the target regardless of who casted it. The following spells will only show times on the buttons if you casted it.",
						order = 1,
					},
					spellname = {
						name = "Spell to add",
						type = "input",
						desc = "Name of the spell to add to the player only buffs list.",
						set = "AddSpellToYourBuff",
						order = 100,
					}, 
				},
			},
			alldebuffs = {
				name = "Any player debuffs",
				desc = "Debuffs for which timers will only be shown regardless of who casted it.",
				type = "group",
				
				args = {
					description = {
						type = "description",
						name = "By default, debuffs will only show times for the target if you were the one to cast it. The following spells will show times on the buttons regardless of who casted it.",
						order = 1,
					},
					spellname = {
						name = "Spell to add",
						type = "input",
						desc = "Name of the spell to add to the any player debuffs list.",
						set = "AddSpellToAnyDebuff",
						order = 100,
					}, 
				},
			},
		},
	}
	
	self:UpdateSpellOptions(self.options.args.ignored.args, self.db.char.ignored, self.ignoreOptions)
	self:UpdateSpellOptions(self.options.args.yourbuffs.args, self.db.char.yourbuffs, self.yourBuffOptions)
	self:UpdateSpellOptions(self.options.args.alldebuffs.args, self.db.char.alldebuffs, self.allDebuffOptions)
end

function Dominos_BuffTimes:UpdateSpellOptions(optionList, spellList, options)
	-- add new spells that aren't in optionList
	for i in pairs(spellList) do
		-- remove space from spell name
		local intName = string.gsub(i, ' ', '')
		intName = "Spell" .. intName
				
		if not optionList[intName] then
			optionList[intName] = {
				name = i,
				type = "group",
				args = options,
			}
		end
	end
	
	-- remove from optionList spells that aren't in spellList
	for i in pairs(optionList) do
		if i ~= "description" and i ~= "spellname" and not spellList[optionList[i].name] then
			optionList[i] = nil
		end
	end
end

function Dominos_BuffTimes:AddSpellToIgnored(info, value)
	self:AddIgnored(value)
	self:UpdateSpellOptions(self.options.args.ignored.args, self.db.char.ignored, self.ignoreOptions)
end

function Dominos_BuffTimes:AddSpellToYourBuff(info, value)
	self:AddYourBuff(value)
	self:UpdateSpellOptions(self.options.args.yourbuffs.args, self.db.char.yourbuffs, self.yourBuffOptions)	
end

function Dominos_BuffTimes:AddSpellToAnyDebuff(info, value)
	self:AddAllDebuff(value)
	self:UpdateSpellOptions(self.options.args.alldebuffs.args, self.db.char.alldebuffs, self.allDebuffOptions)
end

function Dominos_BuffTimes:RemoveSpellFromIgnored(info)
	self:RemoveIgnored(self.options.args.ignored.args[info[#info - 1]].name)
	self:UpdateSpellOptions(self.options.args.ignored.args, self.db.char.ignored, ignoreOptions)
end

function Dominos_BuffTimes:RemoveSpellFromYourBuff(info)
	self:RemoveYourBuff(self.options.args.yourbuffs.args[info[#info - 1]].name)
	self:UpdateSpellOptions(self.options.args.yourbuffs.args, self.db.char.yourbuffs, yourBuffOptions)
end

function Dominos_BuffTimes:RemoveSpellFromAnyDebuff(info)
	self:RemoveAllDebuff(self.options.args.alldebuffs.args[info[#info - 1]].name)
	self:UpdateSpellOptions(self.options.args.alldebuffs.args, self.db.char.alldebuffs, allDebuffOptions)
end

--[[ Slash Commands ]]--

function Dominos_BuffTimes:RegisterSlashCommands()
	self:RegisterChatCommand('bufftimes', 'OnCmd')
end

function Dominos_BuffTimes:OnCmd(args)
	local cmd = string.split(' ', args):lower() or args:lower()
	local restOfString = string.gsub(args, cmd .. ' ', '')

	--frame functions
	if cmd == 'ignored' then
		self:ShowIgnored()
	elseif cmd == 'yourbuffs' then
		self:ShowYourBuffs()
	elseif cmd == 'alldebuffs' then
		self:ShowAllDebuffs()
	elseif cmd == 'addignored' then
		self:AddIgnored(restOfString)
	elseif cmd == 'removeignored' then
		self:RemoveIgnored(restOfString)
	elseif cmd == 'addyourbuff' then
		self:AddYourBuff(restOfString)
	elseif cmd == 'removeyourbuff' then
		self:RemoveYourBuff(restOfString)
	elseif cmd == 'addalldebuff' then
		self:AddAllDebuff(restOfString)
	elseif cmd == 'removealldebuff' then
		self:RemoveAllDebuff(restOfString)
	elseif cmd == 'help' or cmd == '?' then
		self:PrintHelp()
	else
		self:PrintHelp()
	end
end

function Dominos_BuffTimes:PrintHelp(cmd)
	local function PrintCmd(cmd, desc)
		DEFAULT_CHAT_FRAME:AddMessage(format(' - |cFF33FF99%s|r: %s', cmd, desc))
	end
	
	self:Print('Commands (/bufftimes)')
	PrintCmd('ignored', 'Show spells that will not show times on the buttons')
	PrintCmd('addignored <spell name>', 'Add spell that will not show times on the buttons')
	PrintCmd('removeignored <spell name>', 'Remove spell that will not show times on the buttons')
	PrintCmd('yourbuffs', 'Show buffs that will only show times on your casted spells')
	PrintCmd('addyourbuff <spell name>', 'Add buff that will only show times on your casted spells')
	PrintCmd('removeyourbuff <spell name>', 'Remove buff that will only show times on your casted spells')
	PrintCmd('alldebuffs', 'Show debuffs that will show times regardless of the caster')
	PrintCmd('addalldebuff <spell name>', 'Add debuff that will show times regardless of the caster')
	PrintCmd('removealldebuff <spell name>', 'Remove debuff that will show times regardless of the caster')
end

function Dominos_BuffTimes:ShowIgnored()
	self:Print('Spells that will not show times on the buttons')
	if next(self.db.char.ignored) then
		for i in pairs(self.db.char.ignored) do
			self:Print(i)
		end
	end
end

function Dominos_BuffTimes:ShowYourBuffs()
	self:Print('Buffs that will only show times for your casted spells')
	if next(self.db.char.yourbuffs) then
		for i in pairs(self.db.char.yourbuffs) do
			self:Print(i)
		end
	end
end

function Dominos_BuffTimes:ShowAllDebuffs()
	self:Print('Spells that will not show times on the buttons')
	if next(self.db.char.alldebuffs) then
		for i in pairs(self.db.char.alldebuffs) do
			self:Print(i)
		end
	end
end

function Dominos_BuffTimes:AddIgnored(spell)
	local lowerSpell = string.lower(spell)
	self.db.char.ignored[lowerSpell] = 1
	self:Print(lowerSpell .. ' will no longer show buff/debuffs times on buttons')
end

function Dominos_BuffTimes:RemoveIgnored(spell)
	local lowerSpell = string.lower(spell)
	self.db.char.ignored[lowerSpell] = nil
	self:Print(lowerSpell .. ' will now show buff/debuffs times on buttons')
end
		
function Dominos_BuffTimes:AddYourBuff(spell)
	local lowerSpell = string.lower(spell)
	self.db.char.yourbuffs[lowerSpell] = 1
	self:Print(lowerSpell .. ' will now only show buff times on buttons when casted by you')
end

function Dominos_BuffTimes:RemoveYourBuff(spell)
	local lowerSpell = string.lower(spell)
	self.db.char.yourbuffs[lowerSpell] = nil
	self:Print(lowerSpell .. ' will now show buff times on buttons regardless of the caster')
end

function Dominos_BuffTimes:AddAllDebuff(spell)
	local lowerSpell = string.lower(spell)
	self.db.char.alldebuffs[lowerSpell] = 1
	self:Print(lowerSpell .. ' will now show debuff times on buttons regardless of the caster')
end

function Dominos_BuffTimes:RemoveAllDebuff(spell)
	local lowerSpell = string.lower(spell)
	self.db.char.alldebuffs[lowerSpell] = nil
	self:Print(lowerSpell .. ' will now only show debuff times on buttons when casted by you')
end

function Dominos_BuffTimes:IsIgnored(spell)
	local lowerSpell = string.lower(spell)
	return self.db.char.ignored[lowerSpell]
end

function Dominos_BuffTimes:IsYourBuff(spell)
	local lowerSpell = string.lower(spell)
	return self.db.char.yourbuffs[lowerSpell]
end

function Dominos_BuffTimes:IsAllDebuff(spell)
	local lowerSpell = string.lower(spell)
	return self.db.char.alldebuffs[lowerSpell]
end